(function (window, rJS, RSVP, jIO, File, Blob, document) {
    "use strict";
    rJS(window)
    .declareAcquiredMethod("notifySubmitting", "notifySubmitting")
    .declareAcquiredMethod("notifySubmitted", "notifySubmitted")
    .declareAcquiredMethod("getSetting", "getSetting")
    .declareAcquiredMethod("jio_get", "jio_get")

    .ready(function () {
      var gadget = this;
      return this.getDeclaredGadget("model")
      .push(function (model) {
        return gadget.changeState({
          model_gadget: model
        });
      });
    })

    .declareMethod("preRenderDocument", function (parent_options) {
        var gadget = this;
        if (!parent_options.jio_key)
          return;
        return gadget.jio_get(parent_options.jio_key)
          .push(function (parent_document) {
              return parent_document;
          });
    })

    .declareMethod("getIndexPage", function (ids) {
      var gadget = this,
          i,
          get_page,
          page_dict = {},
          get_queue = new RSVP.Queue();

      get_page = function (page_id) {
        get_queue
          .push(function () {
            return gadget.state.model_gadget.get_page(page_id);
          })
          .push(function (page) {
            page_dict[page_id] = page;
          });
      };

      // get all page of this index
      for (i = 0; i < ids.length; i += 1) {
        get_page(ids[i].slice(1));
      }
      return get_queue
        .push(function () {
          return page_dict;
        });
    })

    .declareMethod("putIndexPage", function (page_dict) {
      var gadget = this,
          key,
          put_page,
          put_queue = new RSVP.Queue();

      put_page = function (page_id, data) {
        put_queue
          .push(function () {
            return gadget.state.model_gadget.put(page_id, data);
          });
      };

      for (key in page_dict) {
        put_page(key, page_dict[key]);
      }
      return put_queue;
    })

    .declareMethod("exportIndex", function (index_doc) {
      var gadget = this,
          ids,
          map,
          ctx,
          index_name = index_doc.title,
          index_id = "index-" + index_name;

      return gadget.state.model_gadget.getAttachment(
        index_id,
        "ids",
        {"format": "json"})
      .push(function (result) {
        ids = result;
        return gadget.state.model_gadget.getAttachment(
          index_id,
          "map",
          {"format": "text"});
      })
      .push(function (result) {
        map = result;
        return gadget.state.model_gadget.getAttachment(
          index_id,
          "ctx",
          {"format": "text"});
      })
      .push(function (result) {
        ctx = result;
        return gadget.getIndexPage(ids);
      })
      .push(function (result) {
        var i,
            file_dict,
            file_blob,
            link,
            url;

        file_dict = {
          name: index_name,
          links: index_doc.links,
          ids: JSON.stringify(ids),
          map: map,
          ctx: ctx,
          page: result
        };

        file_blob = new Blob([JSON.stringify(file_dict)], {type : "application/json"});

        link = document.createElement("a");
        link.style = "display: none";
        document.body.appendChild(link);
        url = window.URL.createObjectURL(file_blob);
        link.href = url;
        link.download = index_id + ".mynij";
        link.click();
        window.URL.revokeObjectURL(url);
      });
    })

    .declareMethod("importIndex", function (dataurl) {
      var gadget = this,
          index_dict,
          doc = {},
          index_id;

      return new RSVP.Queue()
        .push(function () {
          return jIO.util.dataURItoBlob(dataurl);
        })
        .push(function (file_blob) {
          return jIO.util.readBlobAsText(file_blob, "application/json");
        })
        .push(function (evt) {
          index_dict = JSON.parse(evt.target.result);
          doc.title = index_dict.name;
          doc.links = index_dict.links;
          index_id = "index-" + doc.title;
          return gadget.state.model_gadget.pushIndexBlob(
            index_id,
            index_dict,
            {});
        })
        .push(function () {
          return gadget.putIndexPage(index_dict.page);
        })
        .push(function () {
          return gadget.state.model_gadget.createIndexDoc(doc);
        })
        .push(function () {
          return gadget.state.model_gadget.pushIndex(index_dict.name);
        });
    })

    .declareMethod("handleSubmit", function (content_dict, parent_options) {
        var return_submit_dict,
        gadget = this,
        jio_key = parent_options.action_options.jio_key;

        return new RSVP.Queue()
        .push(function(){
            return gadget.notifySubmitting();
        })
        .push(function(){
            return gadget.getSetting('index_list', []);
        })
        .push(function(index_list){
            var i,
                built = false,
                index = parent_options.doc,
                redirect = {
                  command: "display",
                  options: {page : "ojs_local_controller", jio_key: jio_key}
                };

            if (content_dict.index_file && content_dict.index_file.url) {
              return gadget.importIndex(content_dict.index_file.url)
                .push(function () {
                return {
                  notify: {
                    message: "Index " + content_dict.index_file.file_name  + " Imported.",
                    status: "success"
                  },
                  redirect: redirect
                };
              })
              .push(undefined, function (e) {
                console.log(e);
                return {
                  notify: {
                    message: "Failed to export index '" + index.title + "'",
                    status: "error"
                  },
                  redirect: redirect
                };
              });
            }

            // try to import
            for (i = 0; i < index_list.length; i += 1) {
              if (index_list[i] === index.title) {
                built = true;
                break;
              }
            }
            if (!built || !index_list.length) {
              return {
                notify: {
                  message: "Index " + index.title  + " is not built.",
                  status: "error"
                },
                redirect: redirect
              };
            }
            return gadget.exportIndex(index)
              .push(function () {
                return {
                  notify: {
                    message: "Index " + index.title  + " Exported.",
                    status: "success"
                  },
                  redirect: redirect
                };
              })
              .push(undefined, function () {
                return {
                  notify: {
                    message: "Failed to export index '" + index.title + "'",
                    status: "error"
                  },
                  redirect: redirect
                };
              });
        });
    });

}(window, rJS, RSVP, jIO, File, Blob, document));